"use strict";

var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};
Object.defineProperty(exports, "__esModule", {
  value: true
});
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const logging_1 = __importDefault(require("../utils/logging"));
const dynamicRequire_1 = __importDefault(require("../utils/dynamicRequire"));
const nodeEnv_1 = __importDefault(require("../utils/nodeEnv"));
const isObject_1 = __importDefault(require("../utils/isObject"));
const merge_1 = __importDefault(require("../utils/merge"));
function loadConfigFile(cwd, cfgFile) {
  let config;
  const filePath = path_1.default.join(cwd, cfgFile);
  if (fs_1.default.existsSync(filePath)) {
    try {
      // 不依赖 `resolve.require`, 因为会循环依赖 config
      const configModule = (0, dynamicRequire_1.default)(filePath);
      // 导出函数
      if (typeof configModule === 'function') {
        config = configModule();
      } else {
        config = configModule;
      }
    } catch (error) {
      logging_1.default.log('parse user config error', error);
    }
    if (!(0, isObject_1.default)(config)) {
      throw new TypeError(`config must be a Object or a function that returns a Object, it received ${config}`);
    }
  }
  return config;
}
/**
 * 获取用户自定义配置
 *
 * @template T Type of config
 * @param cwd 当前目录, 默认`process.cwd()`
 * @param cfgFiles 配置文件名列表, 默认`['project.config.js']`
 * @param ignoreNodeEnv 忽略 `NODE_ENV` 检查，默认会判断 `NODE_ENV` 为 `development` 时才读取本地配置
 * @returns
 */
function getUserConfig(cwd = process.cwd(),
// TODO support ts & esm format config file
cfgFiles = [/* 'project.config.ts',  */'project.config.js'], ignoreNodeEnv = false) {
  let config;
  for (let i = 0; i < cfgFiles.length; i++) {
    const cfgFile = cfgFiles[i];
    config = loadConfigFile(cwd, cfgFile);
    let localConfig;
    // 只在开发模式才读取本地配置，避免本地配置影响生产模式构建
    // 本地配置可能在生产模式下出现预期以外的问题，比如配置了只在本地能访问的接口地址
    if (ignoreNodeEnv || nodeEnv_1.default.isDev()) {
      // 读取本地配置 `*.local.js`
      const extname = path_1.default.extname(cfgFile);
      const localCfgFile = cfgFile.replace(new RegExp(`\\${extname}$`), `.local${extname}`);
      localConfig = loadConfigFile(cwd, localCfgFile);
    }
    if (config && localConfig) {
      // 合并本地配置
      config = (0, merge_1.default)(config, localConfig);
    } else if (!config && localConfig) {
      // 只存在本地配置, 直接使用本地配置
      config = localConfig;
    }
    if (config) {
      break;
    }
  }
  return config;
}
exports.default = getUserConfig;