"use strict";

var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};
Object.defineProperty(exports, "__esModule", {
  value: true
});
const path_1 = __importDefault(require("path"));
const getTsConfig_1 = __importDefault(require("../utils/getTsConfig"));
const getMiniProject_1 = __importDefault(require("../utils/getMiniProject"));
/**
 * 去除目录路径尾部斜线
 *
 * @param projectRoot
 * @param dir 相对路径目录路径
 * @returns 绝对路径目录路径，不包含尾部斜线
 */
function removeTrailingSlash(projectRoot, dir) {
  if (dir) {
    // `path.join`会保留尾部斜线
    dir = path_1.default.join(projectRoot, dir);
    // 去除尾部斜线, `path.basename`会去除尾部斜线
    dir = path_1.default.join(path_1.default.dirname(dir), path_1.default.basename(dir));
  }
  return dir;
}
/**
 * 解析构建配置选项`outputDir`, 检查tsconfig `outDir`是否和mini.project.json `miniprogramRoot`或者`pluginRoot`一致,
 * 如果两者同时存在则和其父目录进行对比。
 *
 * @param projectRoot 项目根目录，默认`process.cwd()`.
 * @returns 返回`miniprogramRoot`或者`pluginRoot`, 如果两者都存在返回其父目录,
 * 如果都不存在则默认`dist`, 相对于项目根目录解析为绝对路径目录格式, 不包含目录尾部斜线。
 */
function resolveOutputDir(projectRoot = process.cwd()) {
  var _a, _b;
  // 解析tsconfig outDir
  const tsConfig = (0, getTsConfig_1.default)(projectRoot);
  const tsOurDir = removeTrailingSlash(projectRoot, (_a = tsConfig === null || tsConfig === void 0 ? void 0 : tsConfig.compilerOptions) === null || _a === void 0 ? void 0 : _a.outDir);
  let miniBaseRoot;
  let miniprogramRoot;
  let pluginRoot;
  // 解析miniprogramRoot和pluginRoot
  const miniProject = (0, getMiniProject_1.default)(projectRoot);
  if (miniProject) {
    miniprogramRoot = removeTrailingSlash(projectRoot, miniProject.miniprogramRoot);
    pluginRoot = removeTrailingSlash(projectRoot, miniProject.pluginRoot);
    if (miniprogramRoot && !pluginRoot) {
      miniBaseRoot = miniprogramRoot;
    } else if (!miniprogramRoot && pluginRoot) {
      miniBaseRoot = pluginRoot;
    } else if (miniprogramRoot && pluginRoot) {
      // miniprogramRoot & pluginRoot父目录
      const miniRootDir = path_1.default.dirname(miniprogramRoot);
      const pluginRootDir = path_1.default.dirname(pluginRoot);
      // 小程序源码目录和插件源码目录必须再同一个父目录下
      if (miniRootDir !== pluginRootDir) {
        throw new Error(`miniprogramRoot: ${miniProject.miniprogramRoot || '-'} ` + `and pluginRoot: ${miniProject.pluginRoot || '-'} must be in a same parent directory`);
      }
      miniBaseRoot = miniRootDir;
    }
  }
  // tsconfig outDir必须和小程序源码目录相同
  if (tsOurDir && miniBaseRoot && tsOurDir !== miniBaseRoot) {
    throw new Error(`tsconfig outDir: ${(_b = tsConfig === null || tsConfig === void 0 ? void 0 : tsConfig.compilerOptions) === null || _b === void 0 ? void 0 : _b.outDir} ` + `and miniBaseRoot(miniprogramRoot, pluginRoot): (${(miniProject === null || miniProject === void 0 ? void 0 : miniProject.miniprogramRoot) || '-'}, ` + `${(miniProject === null || miniProject === void 0 ? void 0 : miniProject.pluginRoot) || '-'}) must be same`);
  }
  // 默认以小程序源码目录为outputDir
  if (miniBaseRoot) {
    return miniBaseRoot;
  }
  // 否则默认输出目录`dist`
  const outputDir = path_1.default.join(projectRoot, 'dist');
  return outputDir;
}
exports.default = resolveOutputDir;