/**
 * 小程序项目配置.
 *
 * @see https://opendocs.alipay.com/mini/framework/project
 * @see https://yuque.antfin.com/tiny-site/framework/projectconfig
 */
interface MiniProject {
    /**
     * 指定小程序源码的相对路径（`app.json`文件所在目录）。
     */
    miniprogramRoot?: string;
    /**
     * 指定插件项目的相对路径（`plugin.json`文件所在目录）。
     */
    pluginRoot?: string;
    /**
     * 编译类型, `miniprogram`普通小程序项目, `plugin`小程序插件项目.
     */
    compileType?: 'miniprogram' | 'plugin';
    /**
     * 是否启用`自定义组件`功能，从`1.14.0`版本开始，自定义组件有了较大改动，支持`component2`相关功能: 新增`onInit`、`deriveDataFromProps`生命周期函数, 支持使用`ref`获取自定义组件实例 。
     */
    component2?: boolean;
    /**
     * 升级基础库版本从`v1.x`到`v2.x`.
     */
    enableAppxNg?: boolean;
    /**
     * 是否开启es6语法转换, 编译`node_modules`模块(es6转es5), 仅基础库`2.0`支持.
     */
    enableNodeModuleBabelTransform?: boolean;
    /**
     * 是否开启`axml`严格语法校验, 构建时能检查出标签未闭合等错误.
     */
    axmlStrictCheck?: boolean;
    /**
     * 是否开启模拟器热更新, 启用小程序模块热更新, 仅基础库`2.0`支持.
     */
    enableHMR?: boolean;
    /**
     * 预处理脚本. 通过预构建，用户可以在 编译前/预览前/上传 前执行自定义命令。
     * 此功能可以用来处理ts/less编译、其他框架转小程序等场景。
     * 真机预览/真机调试前将执行`beforePreview`钩子.
     * IDE上传前将执行`beforeUpload`钩子.
     * 雨燕环境一方小程序构建时，会执行`beforePreview`.
     *
     * @see https://yuque.antfin.com/tiny-site/framework/projectconfig#c8789a5a
     */
    scripts?: {
        watch?: string;
        beforeCompile?: string;
        beforePreview?: string;
        beforeUpload?: string;
        [prop: string]: string | undefined;
    };
    /**
     * 打包黑名单. 小程序上传时，会将本地源码打包传到云端进行构建，除了资源文件外，源码包还会包含以下内容:
     *
     * - 小程序源码文件, `.acss`, `.axml`, `.js`, `.json`, `.sjs`.
     * - 依赖包, `node_modules`目录.
     *
     * 如果源码包经过zip压缩后，包大小仍然超过IDE的阈值(当前为`20M`)，上传时会报“包大小超限”的错误。
     */
    exclude?: string[];
    /**
     * 调试/联调的额外配置，生产构建时不生效.
     */
    debugOptions?: {
        /**
         * 是否启用调试配置.
         */
        enable?: boolean;
        /**
         * 静态插件联调配置.
         *
         * @see https://opendocs.alipay.com/mini/plugin/01phjs#%E9%9D%99%E6%80%81%E6%8F%92%E4%BB%B6
         */
        plugins?: Record<string, {
            /**
             * 插件的版本标识.
             */
            version: string;
            [prop: string]: string | undefined;
        }>;
        /**
         * 动态插件联调配置（格式同 plugins）.
         *
         * @see https://opendocs.alipay.com/mini/plugin/01phjs#%E5%8A%A8%E6%80%81%E5%8A%A0%E8%BD%BD%E6%8F%92%E4%BB%B6
         */
        dynamicPlugins?: Record<string, {
            /**
             * 插件的版本标识.
             */
            version: string;
            [prop: string]: string | undefined;
        }>;
    };
}
/**
 * 环境模式对象，可通过接口合并扩展.
 */
interface EnvMap {
    /**
     * 开发环境.
     */
    dev: 'dev';
    /**
     * 测试环境.
     */
    test: 'test';
    /**
     * 预发环境.
     */
    pre: 'pre';
    /**
     * 生产环境.
     */
    prod: 'prod';
}
/**
 * 环境模式, dev: 开发, test: 测试, pre: 预发, prod: 生产.
 */
declare type Env = keyof EnvMap;
/**
 * 项目构建公共配置类型.
 *
 * @see https://www.yuque.com/docs/share/952bf490-5205-4ed7-8c74-d90329162f39?#《@whale.io/mini-cli-common》
 */
interface Config {
    /**
     * 项目根路径, 绝对路径格式, 默认`process.cwd()`.
     */
    projectRoot: string;
    /**
     * 源码目录相对路径，默认`src`，如果为绝对路径则相对项目根目录解析为相对路径，会统一去除头部`./`和尾部斜线`/`。
     */
    sourceDir: string;
    /**
     * 构建输出目录, 一般不用设置, 默认读取mini.project.json `miniprogramRoot`或者`pluginRoot`, 如果两者都存在则解析其其父目录,
     * 如果都不存在则默认`dist`, 相对于项目根目录解析为绝对路径目录格式, 不包含目录尾部斜线。
     */
    outputDir: string;
    /**
     * `copy-node-modules`没处理本地文件依赖形式npm包，配置需要手动复制的本地依赖包，格式为npm包名数组。
     * 注意：如果本地依赖包有依赖，需要手动配置本地包所依赖的包。
     */
    localPkgs?: string[];
}
/**
 * 复合配置类型，支持函数返回配置对象。
 *
 * @template T Type of config
 */
declare type MixedConfig<T = Config> = Partial<T> | (() => Partial<T>);
/**
 * npm `package.json` type
 *
 * @see https://docs.npmjs.com/cli/v9/configuring-npm/package-json
 */
interface PackageJson {
    /** package name */
    name: string;
    /** package version */
    version: string;
    /** package description */
    description?: string;
    /**
     * The "author" is one person. "contributors" is an array of people. A "person" is an object with a
     * "name" field and optionally "url" and "email", like this:
     *
     * ```json
     * {
     *   "name" : "Barney Rubble",
     *   "email" : "b@rubble.com",
     *   "url" : "http://barnyrubble.tumblr.com/"
     * }
     * ```
     *
     * Or you can shorten that all into a single string, and npm will parse it for you:
     *
     * ```json
     * {
     *   "author": "Barney Rubble <b@rubble.com> (http://barnyrubble.tumblr.com/)"
     * }
     * ```
     */
    author?: string | {
        name: string;
        email?: string;
        url?: string;
    };
    /**
     * The main field is a module ID that is the primary entry point to your program. That is, if your package is
     * named `foo`, and a user installs it, and then does `require("foo")`, then your main module's exports object will be returned.
     *
     * This should be a module relative to the root of your package folder.
     *
     * For most modules, it makes the most sense to have a main script and often not much else.
     *
     * If `main` is not set it defaults to `index.js` in the package's root folder.
     */
    main?: string;
    /**
     * A lot of packages have one or more executable files that they'd like to install into the PATH.
     * npm makes this pretty easy (in fact, it uses this feature to install the "npm" executable.)
     */
    bin?: string | Record<string, string>;
    /**
     * The optional `files` field is an array of file patterns that describes the entries to be included when
     * your package is installed as a dependency.
     */
    files?: string[];
    /**
     * The `scripts` property is a dictionary containing script commands that are run at various times in the lifecycle of your package.
     * The key is the lifecycle event, and the value is the command to run at that point.
     */
    scripts?: Record<string, string>;
    /**
     * Dependencies are specified in a simple object that maps a package name to a version range.
     * The version range is a string which has one or more space-separated descriptors.
     * Dependencies can also be identified with a tarball or git URL.
     */
    dependencies?: Record<string, string>;
    /**
     * If someone is planning on downloading and using your module in their program, then they probably don't want
     * or need to download and build the external test or documentation framework that you use.
     *
     * In this case, it's best to map these additional items in a `devDependencies` object.
     */
    devDependencies?: Record<string, string>;
    /**
     * In npm versions 3 through 6, `peerDependencies` were not automatically installed, and would raise a warning
     * if an invalid version of the peer dependency was found in the tree. As of npm v7, peerDependencies are installed by default.
     */
    peerDependencies?: Record<string, string>;
    keywords?: string[];
    /**
     * You should specify a license for your package so that people know how they are permitted to use it,
     * and any restrictions you're placing on it.
     */
    license?: string;
    /** The url to the project homepage. */
    homepage?: string;
}
/**
 * 小程序应用配置 `app.json` 类型定义，小程序应用配置用于对小程序进行全局配置，设置页面文件的路径、窗口表现、多 tab、分包、插件等
 *
 * @see https://opendocs.alipay.com/mini/framework/app-json
 */
interface AppJson {
    /**
     * 指定小程序的默认启动路径（首页）。如果不填，将默认为 `pages` 列表的第一项。不支持带页面路径参数。
     * 注意：此特性从基础库 `2.7.20`，IDE `3.1.2` 开始支持。若强依赖此特性，建议设置最低基础库版本号为 `2.7.20`。否则，在低版本的基础库，会因为无法识别正确的首页而导致渲染出 `返回首页` 图标。
     */
    entryPagePath?: string;
    /**
     * 设置页面路径，页面路径不需要写任何后缀，框架会自动去加载同名的 `.json`、`.js`、`.axml`、`.acss` 文件
     */
    pages: string[];
    /**
     * 是否启用动态插件，声明小程序需要使用 [动态插件](https://opendocs.alipay.com/mini/plugin/plugin-usage#%E5%8A%A8%E6%80%81%E5%8A%A0%E8%BD%BD)，
     * 基础库 `1.22.4` 及以上，支付宝客户端 `10.1.85` 及以上开始支持
     */
    useDynamicPlugins?: boolean;
    /**
     * 静态插件配置，声明小程序需要使用的 [静态插件](https://opendocs.alipay.com/mini/plugin/plugin-usage#%E9%9D%99%E6%80%81%E5%A3%B0%E6%98%8E)，
     * 基础库 `1.22.4` 及以上，支付宝客户端 `10.1.85` 及以上开始支持
     */
    plugins?: Record<string, {
        /** 插件的版本标识，设置 `*` 自动选择版本，设置 `dev:xxx` 快捷联调插件版本号用于联调，如 `dev:0.2.2102081815.36` */
        version: string;
        /** 插件 id */
        provider: string;
    }>;
    /**
     * 设置全局自定义组件声明，在 `app.json` 中声明的自定义组件将会认为成全局自定义组件，在小程序各页面或自定义组件中可以直接使用无需额外声明。
     * 注意：IDE `3.1.2` 及以上开始支持。该功能声明的组件将要被所有页面和组件依赖，可能会影响性能，且会占用主包大小，建议开启 `app.lazyCodeLoading`。
     */
    usingComponents?: Record<string, string>;
}
export type { MiniProject, EnvMap, Env, Config, MixedConfig, PackageJson, AppJson };
