"use strict";

var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};
Object.defineProperty(exports, "__esModule", {
  value: true
});
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const tar_1 = __importDefault(require("tar"));
const config_1 = __importDefault(require("../config"));
const logging_1 = __importDefault(require("../utils/logging"));
const resolve_1 = __importDefault(require("../utils/resolve"));
/**
 * 打包构建输出目录为压缩文件，默认读取`package.json`中`name`作为压缩文件名输出到项目根目录。
 *
 * @param opts 打包选项，`name`打包名，可为绝对路径名或者文件basename，可带后缀名或者不带，如果不带默认`.tgz`，默认按照`package.json`中`name`字段作为压缩文件名。
 * `cwd` 打包的基目录，默认`config.outputDir`，建议绝对路径。
 * @param fileList 打包的文件或者目录列表，默认`['.']`，即打包基目录下所有文件和子目录。
 * @param callback 透传给`tar`，用于异步回调。
 */
function pack(opts, fileList = ['.'], callback) {
  opts = opts || {};
  let {
    name,
    cwd,
    prefix
  } = opts;
  const {
    gzip = true,
    sync = true,
    file
  } = opts;
  if (!prefix &&
  // 只有配置`.`打包基目录下所有文件和子目录时，处理默认前缀
  fileList.length === 1 && fileList[0] === '.') {
    // 读取`package.json` name字段
    if (!name) {
      const pkg = JSON.parse(fs_1.default.readFileSync((0, resolve_1.default)('package.json'), 'utf8'));
      name = pkg.name;
      if (name) {
        // 替换包名特殊符号，按照npm pack生成的tgz格式规则替换
        name = name.replace(/@/g, '');
        name = name.replace(/(\/|\.)/g, '-');
      }
    }
    if (name) {
      // 使用name最后一部分去除后缀名作为prefix
      const basename = path_1.default.basename(name);
      const extname = path_1.default.extname(name);
      prefix = basename;
      // 去除后缀名
      if (extname) {
        prefix = basename.replace(new RegExp(`\\${extname}$`), '');
      }
    }
  }
  if (!cwd) {
    cwd = config_1.default.outputDir;
  }
  let packFile = file;
  if (!packFile) {
    if (!name) {
      throw new Error('name must be given');
    }
    // 添加后缀名
    let filename = name;
    if (!path_1.default.extname(filename)) {
      filename += '.tgz';
    }
    packFile = (0, resolve_1.default)(filename);
  } else {
    packFile = (0, resolve_1.default)(packFile);
  }
  logging_1.default.log('pack folder', path_1.default.relative(config_1.default.projectRoot, cwd));
  logging_1.default.log('to file', path_1.default.relative(config_1.default.projectRoot, packFile));
  if (callback) {
    return tar_1.default.create({
      cwd,
      gzip,
      prefix,
      file: packFile
    }, fileList, callback);
  }
  return tar_1.default.create({
    cwd,
    gzip,
    sync,
    prefix,
    file: packFile
  }, fileList);
}
exports.default = pack;